/*
 * $Header: /cvsroot/xwinx/xwinx/xwxdm.c,v 1.7 2001/11/14 04:56:55 snaund Exp $
 * xwxdm.c - XWinX Display Manager, XDMCP server for initiating XWinX remotely
 * Copyright (C) 2001 Kim Saunders, Sanjay Rao
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <X11/Xlib.h>
#include <X11/Xmd.h>
#include <X11/Xdmcp.h>
#include "xwxmemory.h"
#include "xwxhostname.h"

/* structure to hold info for a session. only the cookie and ID are stored
 * at the moment, but in future we may wish to store the session start time,
 * display number, etc. not sure, just used a struct for flexibility basically.
 */
struct xwxdm_session
{
	unsigned char cookie[16];	/* MIT-MAGIC-COOKIE-1 session cookie */
	unsigned int id;		/* session ID */
};

/* file descriptor for socket */
static int fd = -1;

/* current session ID counter */
unsigned int sess_id;

/* sess_db is an array pointers to xwxdm_session structs */
struct xwxdm_session **sess_db;

typedef struct _XdmAuth
{
	ARRAY8 authentication;
	ARRAY8 authorization;
} XdmAuthRec, *XdmAuthPtr;

static XdmAuthRec slist = 
{
	{ (CARD16) 0, (CARD8 *) 0 },
	{ (CARD16) 0, (CARD8 *) 0 }
};

/* XdmcpBuffer for reading incoming xdmcp packets into */
XdmcpBuffer buffa;

/* XdmcpHeader to store the header of incoming packets in */
XdmcpHeader hedda;

/* sockaddr_in structure for the client connection */
struct sockaddr_in	client_sa;

/* string containing the IP address of the client, as a dotted quad */
char			*client_ip;

/* write the packet written to buffa to fd using XdmcpFlush */
void xwxdm_flush ()
{
	if (
		!XdmcpFlush(
			fd,
			&buffa,
			(XdmcpNetaddr)&client_sa,
			(int)sizeof(struct sockaddr_in)
		)
	)
		fprintf(stderr, "\tERROR: Can't flush XDMCP packet.\n");
}

/* sets the data member of an ARRAY8 and calculate and set the length member */
void set_ARRAY8(ARRAY8 *aeight, char *string)
{
	aeight->data = string;
	aeight->length = strlen(aeight->data);
}

/* create and send a WILLING packet */
void send_willing (ARRAY8Ptr hostname, ARRAY8Ptr status)
{
	/* header for outgoing WILLING packet */
	XdmcpHeader	willing;

	/* set up the header for the packet */
	willing.opcode = (CARD16) WILLING;
	willing.length = 6;
	willing.length += slist.authentication.length;
	willing.length += hostname->length;
	willing.length += status->length;
	willing.version = XDM_PROTOCOL_VERSION;

	/* write the packet into buffa */
	XdmcpWriteHeader(&buffa, &willing);
	XdmcpWriteARRAY8(&buffa, &slist.authentication);
	XdmcpWriteARRAY8(&buffa, hostname);
	XdmcpWriteARRAY8(&buffa, status);

	/* send the packet */
	xwxdm_flush();
}

void send_accept (
	CARD32Ptr session,
	ARRAY8Ptr authen_name,
	ARRAY8Ptr authen_data,
	ARRAY8Ptr author_name,
	ARRAY8Ptr author_data
)
{
	XdmcpHeader	accept;
	
	/* set up header for outgoing ACCEPT packet */
	accept.opcode = (CARD16) ACCEPT;
	accept.length = 12;
	accept.length += authen_name->length;
	accept.length += authen_data->length;
	accept.length += author_name->length;
	accept.length += author_data->length;
	accept.version = XDM_PROTOCOL_VERSION;

	/* write packing into buffa */

	/* packet header */
	XdmcpWriteHeader(&buffa, &accept);

	/* session ID number */
	XdmcpWriteCARD32(&buffa, *session);

	/* authentication name and data */
	XdmcpWriteARRAY8(&buffa, authen_name);
	XdmcpWriteARRAY8(&buffa, authen_data);

	/* authorisation name and data */
	XdmcpWriteARRAY8(&buffa, author_name);
	XdmcpWriteARRAY8(&buffa, author_data);

	/* send packet to client */
	xwxdm_flush();
}

/* respond to a QUERY packet that has been written info buffa */
void resp_query ()
{
	ARRAY8 host, stat;
	ARRAYofARRAY8 list;

	/* we don't actually use the auth list for anything, indeed it's
	 * usually empty, but we just read it to make sure the packet is
	 * properly formed.
	 */
	if (!XdmcpReadARRAYofARRAY8(&buffa, &list))
	{
		XdmcpDisposeARRAYofARRAY8(&list);
		fprintf(stderr, "\tERROR: Can't read auth list.\n");
		return;
	}

	/* setup ARRAY8s for the hostname and status strings to send back */
	set_ARRAY8(&host, xwxhostname_gethostname());
	set_ARRAY8(&stat, strdup("XWinX"));

	/* send a WILLING packet back to the client. We're not fussy at the
	 * moment, sending a WILLING packet to anyone who sends a QUERY to us.
	 * At some point, I guess I'll add some sort of ACL to make sure the
	 * client IP address is legal, but at this stage XDMCP is bare-bones
	 */
	send_willing(&host, &stat);
	
	/* this frees memory pointed to these structures */
	XdmcpDisposeARRAY8(&host);
	XdmcpDisposeARRAY8(&stat);
	XdmcpDisposeARRAYofARRAY8(&list);
}

/* respond to a REQUEST packet in buffa */
void resp_request ()
{
	/* client display number, manufacturer and authorisation names */
	CARD16			cli_dpynumb;
	ARRAY8			cli_dpymanu;
	ARRAYofARRAY8		cli_authnames;
	
	/* server authentication, authorisation names and data */
	ARRAY8			srv_authenname;
	ARRAY8			srv_authendata;
	ARRAY8			srv_authorname;
	ARRAY8			srv_authordata;

	/* pointer so a xwxdm session struct to store session details in */
	struct xwxdm_session	*session;

	/* array of bytes to store 128 boot MIT-MAGIC-COOKIE-1 cookie in */
	unsigned char		*cookie_dat;

	/* int for loops */
	int			i;

	/* temp variables to read things into then discard */
	ARRAY8			tmp_ARRAY8;
	ARRAY16			tmp_ARRAY16;
	ARRAYofARRAY8		tmp_ARRAYofARRAY8;

	/* read all of the received packet fields. Again, not many of these
	 * actually matter to xwxdm, but we read them anyway to make sure the
	 * packet is properly formed with all the fields, and so we can print
	 * them for debugging if necessary
	 */

	/* this is the display number. keep this, relatively interesting */
	if (!XdmcpReadCARD16 (&buffa, &cli_dpynumb))
	{
		fprintf(stderr, "\tERROR: Can't read display number.\n");
		return;
	}

	/* these are connection types and connection addresses. NFI what it's
	 * about, so just read it into a temp variable to make sure it's
	 * there, then discard
	 */
	if (!XdmcpReadARRAY16 (&buffa, &tmp_ARRAY16))
	{
		fprintf(stderr, "\tERROR: Can't read connection type.\n");
		XdmcpDisposeARRAY16(&tmp_ARRAY16);
		return;
	}
	XdmcpDisposeARRAY16(&tmp_ARRAY16);

	if (!XdmcpReadARRAYofARRAY8 (&buffa, &tmp_ARRAYofARRAY8))
	{
		fprintf(stderr, "\tERROR: Can't read client addresses.\n");
		XdmcpDisposeARRAYofARRAY8(&tmp_ARRAYofARRAY8);
		return;
	}
	XdmcpDisposeARRAYofARRAY8(&tmp_ARRAYofARRAY8);

	/* this is for the X display to authenticate itself with us. neither
	 * xwxdm nor any other display managers seem to support this, so same
	 * deal, read it to make sure we can, then discard.
	 */
	if (!XdmcpReadARRAY8 (&buffa, &tmp_ARRAY8))
	{
		fprintf(stderr, "\tERROR: Can't read authentication name.\n");
		XdmcpDisposeARRAY8(&tmp_ARRAY8);
		return;
	}
	XdmcpDisposeARRAY8(&tmp_ARRAY8);

	if (!XdmcpReadARRAY8 (&buffa, &tmp_ARRAY8))
	{
		fprintf(stderr, "\tERROR: Can't read authentication data.\n");
		XdmcpDisposeARRAY8(&tmp_ARRAY8);
		return;
	}
	XdmcpDisposeARRAY8(&tmp_ARRAY8);
	
	/* this is an array of authorisation names that the X server accepts.
	 * This is important, we need to look at it, and use MIT-MAGIC-COOKIE-1
	 * if possible
	 */
	if (!XdmcpReadARRAYofARRAY8 (&buffa, &cli_authnames))
	{
		fprintf(stderr, "\tERROR: Can't read auth names.\n");
		XdmcpDisposeARRAYofARRAY8(&cli_authnames);
		return;
	}

	/* the manufacturer string for the client. not important, but human
	 * readable, so we read it and print it out.
	 */
	if (!XdmcpReadARRAY8 (&buffa, &cli_dpymanu))
	{
		fprintf(stderr, "\tERROR: Can't read display manufacturer.\n");
		XdmcpDisposeARRAYofARRAY8(&cli_authnames);
		XdmcpDisposeARRAY8(&cli_dpymanu);
		return;
	}

	/* print the display manufacturer and display number out, it's vaguely
	 * intersting...
	 */
	printf("\tDisplay number: %d\n", cli_dpynumb);
	printf("\tDisplay manufacturer: %s\n", cli_dpymanu.data);

	/* printed the manufacturer sting out so it can be discarded now */
	XdmcpDisposeARRAY8(&cli_dpymanu);

	/* go throught the authorisation names supported by the display */
	for (i = 0; i < cli_authnames.length; i++)
	{
		int j;

		/* XDMCP fields aren't neccesarily null terminated, so print
		 * out the name char by char instead of as a string
		 */
		printf("\tAuthorisation: ");
		for (j = 0; j < cli_authnames.data[i].length; j++)
			printf("%c", cli_authnames.data[i].data[j]);
		printf("\n");
	}

	/* set the authentication type and data to null. Authentication
	 * isn't supported by xwxdm, and doesn't seem to be used by anything
	 * else I've seen (gdm certainly doesn't use it)
	 */
	set_ARRAY8(&srv_authenname, strdup(""));
	set_ARRAY8(&srv_authendata, strdup(""));

	/* set up MIT-MAGIC-COOKIE-1 authorisation */
	set_ARRAY8(&srv_authorname, strdup("MIT-MAGIC-COOKIE-1"));
	
	/* ultra-secure random seed... */
	srand(getpid() + time(NULL));

	/* allocate 16 bytes (128 bits) for the cookie */
	cookie_dat = (unsigned char *)xwxmalloc(16);

	/* set each byte of cookie_dat to a random number between 0 and 255 */
	for (i = 0; i < 16; i++)
		cookie_dat[i] = rand() % 256;
	
	/* don't use set_ARRAY8 since this is just an array of bytes, not a
	 * null terminated string
	 */
	srv_authordata.data = (CARD8Ptr)cookie_dat;
	srv_authordata.length = 16;

	/* send an ACCEPT packet to the client. xwxdm is still fairly
	 * primitive, we should be sending decline in some circumstances
	 * (dodgey packet, request that doesn't follow a query, etc)
	 */
	send_accept(
		(CARD32Ptr)&sess_id,
		&srv_authenname,
		&srv_authendata,
		&srv_authorname,
		&srv_authordata
	);

	/* reallocate the sess_db array to hold another pointer */
	sess_db = xwxrealloc(
		sess_db,
		sizeof(struct xwxdm_session *) * (sess_id + 1)
	);

	/* allocate memory for an xwxdm_session struct */
	session = (struct xwxdm_session *)xwxmalloc(
		sizeof(struct xwxdm_session)
	);
	
	/* set the members to the session structure */
	session->id = sess_id;
	memcpy(&(session->cookie), cookie_dat, 16);

	/* parcket has been send, discard all the ARRAY8s */
	XdmcpDisposeARRAY8(&srv_authenname);
	XdmcpDisposeARRAY8(&srv_authendata);
	XdmcpDisposeARRAY8(&srv_authorname);
	XdmcpDisposeARRAY8(&srv_authordata);

	sess_db[sess_id] = session;

	sess_id++;
}

void resp_manage ()
{
	/* session and display number from display */
	CARD32			man_session;
	CARD16			man_display;

	/* command to run */
	char			*run_str;
	int			run_len;

	/* a session to dig out from sess_db */
	struct xwxdm_session	*session;

	/* counter for loops */
	int			i;

	/* return value from snprintf */
	int			siz;

	/* string to hold 32 char hex string for cookie */
	char			chex[33];

	/* pid for forking */
	pid_t			pid;
	
	if (!XdmcpReadCARD32(&buffa, &man_session))
	{
		fprintf(stderr, "\tERROR: Can't read session ID.\n");
		return;
	}

	if (!XdmcpReadCARD16(&buffa, &man_display))
	{
		fprintf(stderr, "\tERROR: Can't read display number.\n");
		return;
	}

	/* session_id is larger than the sessions xwxdm knows about. Ideally,
	 * xwxdm should send a REFUSE packet
	 */
	if ( !( (int)man_session < sess_id ) )
		return;

	/* fetch the relevant session record from the array */
	session = sess_db[ (int)man_session ];
	
	/* construct a string containing the hex for the 128 bit cookie */
	for (i = 0; i < 16; i++)
		sprintf((char *)&chex[i * 2], "%02x", session->cookie[i]);

	/* null terminator for string */
	chex[32] = 0;

	/* allocate memory for the run string */
	run_len = 2;
	run_str = (char *)xwxmalloc(run_len);
	siz = 0;
	while (1)
	{
		/* attempt to write the string into run_str */
		siz = snprintf(
			run_str,
			run_len,
			"./xwinx.exe -d %s:%d -c %s",
			client_ip,
			man_display,
			chex
		);
		
		/* snprintf returned -1, meaning the buffer wasn't big enough,
		 * so we shall double it and try again
		 */
		if (siz < 0)
		{
			run_len *= 2;
			run_str = (char *)xwxrealloc(run_str, run_len);
			continue;
		}
		
		/* snprintf returned a length longer than our buffer, which
		 * means the buffer needs to be as long as the return value,
		 * plus one for the null terminator. realloc and try again.
		 */
		if (siz > run_len)
		{
			run_len = siz + 1;
			run_str = (char *)xwxrealloc(run_str, run_len);
			continue;
		}

		/* if we got this far, it's been written to run_str, break */
		break;
	}

	printf("\tStarting XWinX (session %d):\n", (int)man_session);
	printf("\t%s\n", run_str);

	/* fork a child process to run xwinx */
	pid = fork();

	/* parent process returns here */
	if (pid)
		return;
		
	/* only the child process will get this far */

	/* run xwinx */
	system(run_str);

	/* exit */
	exit(EXIT_SUCCESS);
}

/* return a name of an opcode as a string */
char *xwxdm_opcode_string (int opcode)
{
	switch (opcode)
	{
		case BROADCAST_QUERY:
			return strdup("BROADCAST_QUERY");
		case QUERY:
			return strdup("QUERY");
		case WILLING:
			return strdup("WILLING");
		case REQUEST:
			return strdup("REQUEST");
		case MANAGE:
			return strdup("MANAGE");
		case ACCEPT:
			return strdup("ACCEPT");
		default:
			return strdup("Unknown packet");
	}
}

/* print a message announcing a packet, and the ip it came from */
void xwxdm_announce (int opcode)
{
	/* a string to store the name of the opcode */
	char *ocstring;

	/* get the string for the opcode */
	ocstring = xwxdm_opcode_string(opcode);

	/* print the announcement */
	printf("%s packet received from %s.\n", ocstring, client_ip);

	/* free the memory allocated for the string name */
	free(ocstring);
}

int main ()
{
	/* a socket to listen on */
	struct sockaddr_in server_sa = {0};

	/* start session ids at 1 */
	sess_id = 1;

	/* allocate enough memory to hold one pointer to a sess_db record */
	sess_db = xwxmalloc(sizeof(void *));

	/* try and open a socket */
	fd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP); 
	if (fd < 0)
	{
		fprintf(stderr, "ERROR: Can't create socket!\n");
		exit(EXIT_FAILURE);
	}

	/* set up the socket to listen on port 177 (XDMCP) */
	server_sa.sin_family = AF_INET;
	server_sa.sin_port = htons((short)177);
	server_sa.sin_addr.s_addr = htonl(INADDR_ANY);	

	/* try and bind the socket to the port */
	if (bind(fd, (struct sockaddr*)&server_sa, sizeof(server_sa)) == -1)
	{
		fprintf(stderr, "ERROR: Can't bind to XDMCP socket!\n");
		exit(EXIT_FAILURE);
	}

	printf("XWXDM: Waiting for XDMCP packets...\n");
	while (1)
	{
		/* length of buffa */
		int buffa_len;

		buffa_len = sizeof(buffa);
		
		/* read a packet from fd into buffa */
		if (	
			!XdmcpFill(
				fd,
				&buffa,
				(XdmcpNetaddr)&client_sa,
				&buffa_len
			)
		)
		{
			fprintf(stderr, "ERROR: XdmcpFill failed.\n");
			continue;
		}

		/* get client IP address string as dotted quad */
		client_ip = inet_ntoa(client_sa.sin_addr);
		
		/* read the XDMCP packet header from buffa */
		if (!XdmcpReadHeader(&buffa, &hedda))	
		{
			fprintf(stderr, "ERROR: Can't read XDMCP header.\n");
			continue;
		}

		/* check the incoming packet is using the same XDMCP version */
		if (hedda.version != XDM_PROTOCOL_VERSION)
		{
			fprintf(stderr, "ERROR: Incompatible XDMCP version.\n");
			continue;
		}

		/* print a message announcing the receipt of the packet */
		xwxdm_announce(hedda.opcode);

		/* respond the the packet, depending on the opcode */
		switch (hedda.opcode)
		{
			case BROADCAST_QUERY:
			case QUERY:	
				resp_query();
				break;
			case REQUEST:
				resp_request();
				break;
			case MANAGE:
				resp_manage();
				break;
		}
		printf("\n");
	}
	return(EXIT_SUCCESS);
}
