/*
 * $Header: /cvsroot/xwinx/xwinx/xwxmemory.c,v 1.5 2001/11/04 13:44:11 snaund Exp $
 * xwxmemory.c - wrappers around memory allocation functions
 * Copyright (C) 2001 Kim Saunders, Sanjay Rao
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <X11/Xlib.h>
#include "xwx.h"
#include "xwxprint.h"
#include "xwxmemory.h"

/* these are basically just wrappers around the normal memory allocation
 * functions that check for errors so that you don't have to do it yourself
 * every single time. I won't bother commenting them individually, they're 
 * pretty straightforward
 */

void *xwxcalloc (size_t nmemb, size_t size)
{
	unsigned int bytes;
	void *mem;

	/* almost all of my calloc's have nmemb==1, so this saves
	 * the multiply in the common case 
	 */
	if (nmemb==1) bytes=size;
	else bytes=nmemb*size;


	mem=xwxmalloc(bytes);
	if (mem) memset(mem,0,bytes);

	return(mem);

}

void *xwxmalloc (size_t size)
{
	void *mem;
	struct params *dummyparams = 0;

	mem = malloc(size);
	if (mem == NULL)
		errMemory(dummyparams);
	
	return(mem);
}

void *xwxrealloc (void *ptr, size_t size)
{
	void *mem;
	struct params *dummyparams = 0;
	
	mem = realloc(ptr, size);
	if (mem == NULL)
		errMemory(dummyparams);

	return(mem);
}

void *xwxstrdup (const char *string)
{
	void *mem;
	struct params *dummyparams = 0;

	mem = strdup(string);
	if (mem == NULL)
		errMemory(dummyparams);

	return(mem);
}
