/*
 * $Header: /cvsroot/xwinx/xwinx/xwxmouse.c,v 1.9 2001/11/08 00:10:36 guilty Exp $
 * xwxmouse.c - Mouse support functions for xwinx 
 * Copyright (C) 2001 Kim Saunders, Sanjay Rao
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <wingdi.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <pthread.h>
#include "xwx.h"
#include "xwxwindows.h"
#include "xwxmouse.h"

/* update the mouse cursor position on the win32 desktop */
void mouUpdate (struct xinput *xinput, struct x11info *x11info){
	Window rrt, crt;	/* root and client windows cursor is in	*/
	int cmx, cmy;		/* client window mouse coords 		*/
	int rmx, rmy;		/* root window mouse coords 		*/
	unsigned int mrt;	/* mouse return 			*/

	/* Get location of X cursor */
	pthread_mutex_lock(&xinput->inputlock);
	XQueryPointer(
		x11info->display,
		x11info->window,
		&rrt,
		&crt,
		&rmx,
		&rmy,
		&cmx,
		&cmy,
		&mrt
	);
	pthread_mutex_unlock(&xinput->inputlock);

	/* SetCursorPos() if the curso has moved */
	if ( (x11info->pmx != cmx) || (x11info->pmy != cmy) ){
		SetCursorPos(cmx, cmy);
		x11info->pmx = cmx;
		x11info->pmy = cmy;
	}
}

/* simulate a corresponding win32 mouse event for an XEvent */
void mouPressEvent(struct x11info *x11info, struct params *params, XEvent evt)
{
	DWORD flg = 0;	/* mouse movement flag */
	DWORD whl = 0;  /* amount of wheel movement */

	/* a mouse event will unpause xwinx if it is paused */
	unpause_xwinx();

	switch (evt.type)
	{
		case ButtonPress:
			switch (evt.xbutton.button)
			{
				case Button1:
					flg = MOUSEEVENTF_LEFTDOWN;
					break;
				case Button2:
					flg = MOUSEEVENTF_MIDDLEDOWN;
					break;
				case Button3:
					flg = MOUSEEVENTF_RIGHTDOWN;
					break;
				case Button4:
					flg = XWX_MOUSEEVENTF_WHEEL;
					whl = WHEEL_DELTA;
					break;
				case Button5:
					flg = XWX_MOUSEEVENTF_WHEEL;
					whl = -WHEEL_DELTA;
					break;
			}
			break;
		case ButtonRelease:
			switch (evt.xbutton.button)
			{
				case Button1:
					flg = MOUSEEVENTF_LEFTUP;
					break;
				case Button2:
					flg = MOUSEEVENTF_MIDDLEUP;
					break;
				case Button3:
					flg = MOUSEEVENTF_RIGHTUP;
					break;
			}
			break;
	}				
	
	/* only send an event if we calculated the flag */
	if (flg)
	{
		/* move the cursor to the pace the event occurred */
		SetCursorPos(evt.xbutton.x, evt.xbutton.y);
		/* send the click/release event */
		mouse_event(flg, 0, 0, whl, (int)NULL);
	}
}
