/*
 * $Header: /cvsroot/xwinx/xwinx/xwxprint.c,v 1.18 2001/11/16 07:05:23 snaund Exp $
 * xwxprint.c - functions to print out text 
 * Copyright (C) 2001 Kim Saunders, Sanjay Rao
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <wingdi.h>
#include <getopt.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <pthread.h>
#include "xwx.h"
#include "xwxprint.h"

/* print xwinx usage information and exit */
void prnHelp (struct params *params)
{
	fprintf(
		stderr,
		"Usage: %s [OPTION]\n"\
		"Project Microsoft Windows desktop to X server over network.\n"\
		"\n"\
		"  -d, --display"\
			"\t\t\tX11 display\n"\
		"  -c, --mit-magic-cookie-1"\
			"\tAuthorisation cookie for X11 display\n"\
		"  -b, --bpp"\
			"\t\t\tDepth of X11 window\n"\
		"  -h, --help"\
			"\t\t\tUsage information\n"\
		"  -v, --version"\
			"\t\t\tVersion and copyright information\n"\
		"  -p, --port"\
			"\t\t\tListen for requests on this port\n"\
		"  -0, --pause"\
			"\t\t\tPause updating (requires port)\n"\
		"  -1, --unpause"\
			"\t\t\tResume for updating\n"\
		"  -a, --auth"\
			"\t\t\tUse user:password for HTTP\n"\
		"  -n, --nodisplay"\
			"\t\tDon't create window (HTTP only)\n",
		params->invname
	);
}

void prnHttpWarnings (struct params *params)
{
	if ((params->httpauth) && (!params->httpport)) {
		fprintf(stderr,"WARNING: HTTPD will not be started "\
			"as no port was specified (use --port).\n\n");
	}
}

/* print xwinx banner */
void prnBanner (struct params *params)
{
	fprintf(
		stderr,
		"%s %s\n"\
		"Copyright (C) %d %s\n",
		XWX_NAME,
		XWX_VERS,
		XWX_YEAR,
		XWX_COPY
	);
}

/* print banner then exit */
void prnVersion (struct params *params)
{
	prnBanner(params);
}

/* print a warning if a keyboard conversion failed */
void wrnKeyboard (struct params *params, char *key)
{
	fprintf(
		stderr,
		"WARNING: %s doesn't support the %s key\n",
		XWX_NAME,
		key
	);
}

/* error allocating memory */
void errMemory (struct params *params)
{
	fprintf(
		stderr,
		"ERROR: Cannot allocate memory.\n"
	);
	exit(EXIT_FAILURE);
}

void errX11Window (struct params *params)
{
	fprintf(
		stderr,
		"ERROR: Cannot create X11 window on display.\n"
	);
}

void errX11Depth (struct params *params)
{
	fprintf(
		stderr,
		"ERROR: XWinX only supports 8, 16 and 24 bit X11 displays.\n"
	);
}

void errX11Cookie (struct params *params)
{
	fprintf(
		stderr,
		"ERROR: Cookie must be 32 character hex number.\n"
	);
	exit(EXIT_FAILURE);
}

void errX11Display (struct params *params)
{
	fprintf(
		stderr,
		"ERROR: Can't open display: %s\n",
		params->display
	);
}

void errX11XImage (struct params *params)
{
	fprintf(
		stderr,
		"ERROR: Can't create XImage.\n"
	);
}

void errGetDIBits (struct params *params)
{
	fprintf(
		stderr,
		"ERROR: The current video driver does not support the \n"\
		"\tmethod we use to query (GetDIBits). Sorry.\n"
	);
}
